﻿/*
Created by: X
http://www.createdbyx.com/

This work is licensed under a Creative Commons Attribution-Noncommercial-Share Alike 2.5 Canada License 
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/. 
Permissions beyond the scope of this license may be available at http://contact.createdbyx.com/.
*/
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.VisualStudio.DebuggerVisualizers;
using Microsoft.Xna.Framework.Graphics;
using XNAVisies;
using Color = Microsoft.Xna.Framework.Graphics.Color;

[assembly: DebuggerVisualizer(typeof(XNAVisies.ColorArrayVisualizer), typeof(ColorArrayDebugTypeVisualizer), Target = typeof(Color[]), Description = "XNAVisies: Color Array Visualizer")]

namespace XNAVisies
{
    public class ColorArrayDebugTypeVisualizer : Microsoft.VisualStudio.DebuggerVisualizers.VisualizerObjectSource
    {
        public ColorArrayDebugTypeVisualizer()
            : base()
        {

        }

        public override void GetData(object target, System.IO.Stream outgoingData)
        {
            System.Diagnostics.Debugger.Launch();
            if (!(target is Color[])) return;
            Color[] value = (Color[])target;

            BinaryWriter writer = new BinaryWriter(outgoingData);
            for (int i = 0; i < value.Length; i++)
            {
                outgoingData.WriteByte(value[i].R);
                outgoingData.WriteByte(value[i].G);
                outgoingData.WriteByte(value[i].B);
                outgoingData.WriteByte(value[i].A);
            }
        }

        public override void TransferData(object target, System.IO.Stream incomingData, System.IO.Stream outgoingData)
        {
            base.TransferData(target, incomingData, outgoingData);
        }

        public override object CreateReplacementObject(object target, System.IO.Stream incomingData)
        {
            return base.CreateReplacementObject(target, incomingData);
        }
    }


    // TODO: Add the following to SomeType's definition to see this visualizer when debugging instances of SomeType:
    //
    // [DebuggerVisualizer(typeof(MatrixVisualizer))]
    // [Serializable]
    // public class SomeType
    // {
    // ...
    // }
    //
    /// <summary>
    /// A Visualizer for SomeType.
    /// </summary>
    public class ColorArrayVisualizer : DialogDebuggerVisualizer
    {

        protected override void Show(IDialogVisualizerService windowService, IVisualizerObjectProvider objectProvider)
        {
            try
            {

                // TODO: Get the object to display a visualizer for.
                // Cast the result of objectProvider.GetObject()
                // to the type of the object being visualized.
                System.Diagnostics.Debugger.Launch();
                var tmp = objectProvider.GetData();
                //var tmp = objectProvider.GetObject();

                System.IO.BinaryReader reader = new BinaryReader(tmp);
                var lst = new List<Color>();

                try
                {
                    while(true)
                    {
                        Color ColValue = new Color();
                        ColValue.R = reader.ReadByte();
                        ColValue.G = reader.ReadByte();
                        ColValue.B = reader.ReadByte();
                        ColValue.A = reader.ReadByte();
                        lst.Add(ColValue);
                    }
                }
                catch (EndOfStreamException e)
                {
                }


                //if (!tmp.GetType().Equals(typeof(List<Color>)))
                //{
                //    MessageBox.Show("The List does not contain Color types.", "Information", MessageBoxButtons.OK,
                //                    MessageBoxIcon.Information);
                //    return;
                //}

                //Color[] Col = ((List<Color>)tmp).ToArray();
                Color[] Col = lst.ToArray();

                // TODO: Display your view of the object.
                // Replace displayForm with your own custom Form or Control.
                using (ImageVisualizerForm displayForm = new ImageVisualizerForm())
                {

                    Bitmap bmp = new Bitmap(512, 512, PixelFormat.Format32bppArgb);


                    // Lock the bitmap's bits.  
                    Rectangle rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
                    System.Drawing.Imaging.BitmapData bmpData =
                        bmp.LockBits(rect, System.Drawing.Imaging.ImageLockMode.ReadWrite,
                                     bmp.PixelFormat);

                    // Get the address of the first line.
                    IntPtr ptr = bmpData.Scan0;

                    // Declare an array to hold the bytes of the bitmap.
                    int bytes = bmpData.Stride * bmp.Height;
                    byte[] rgbValues = new byte[bytes];

                    // Copy the RGB values into the array.
                    System.Runtime.InteropServices.Marshal.Copy(ptr, rgbValues, 0, bytes);


                    // copy pixel data into bitmat
                    int index = 0;
                    for (int i = 0; i < rgbValues.Length; i += 4)
                    {
                        rgbValues[i + 2] = Col[index].B;
                        rgbValues[i + 1] = Col[index].G;
                        rgbValues[i] = Col[index].R;
                        rgbValues[i + 3] = Col[index].A;
                        index++;
                    }

                    // Copy the RGB values back to the bitmap
                    System.Runtime.InteropServices.Marshal.Copy(rgbValues, 0, ptr, bytes);

                    // Unlock the bits.
                    bmp.UnlockBits(bmpData);


                    // assign the matrix
                    displayForm.Image = bmp;

                    if (windowService.ShowDialog(displayForm) == DialogResult.OK)
                    {

                        // update object values
                        //      objectProvider.ReplaceData(displayForm.GetImageData());
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("An unexpected error occoured!\r\n\r\n" + ex.Message + "\r\n\r\n" + ex.StackTrace, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // TODO: Add the following to your testing code to test the visualizer:
        //
        // MatrixVisualizer.TestShowVisualizer(new SomeType());
        //
        /// <summary>
        /// Tests the visualizer by hosting it outside of the debugger.
        /// </summary>
        /// <param name="objectToVisualize">The object to display in the visualizer.</param>
        public static void TestShowVisualizer(object objectToVisualize)
        {
            VisualizerDevelopmentHost visualizerHost = new VisualizerDevelopmentHost(objectToVisualize, typeof(ColorVisualizer));
            visualizerHost.ShowVisualizer();
        }
    }
}
