﻿/*
Created by: X
http://www.createdbyx.com/

This work is licensed under a Creative Commons Attribution-Noncommercial-Share Alike 2.5 Canada License 
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/. 
Permissions beyond the scope of this license may be available at http://contact.createdbyx.com/.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework.Graphics;

namespace XNAVisies
{
    public partial class ImageVisualizerForm : Form
    {
        private SurfaceFormat format = SurfaceFormat.Unknown;

        public SurfaceFormat Format
        {
            private get { return this.format; }
            set
            {
                this.format = value;
                this.lblFormat.Text = "Format: " + value.ToString();
            }
        }

        public Bitmap Image
        {
            get { return (Bitmap)this.pbPreview.Image; }
            set
            {
                this.pbPreview.Image = value;
                this.txtWidth.Text = value.Width.ToString();
                this.txtHeight.Text = value.Height.ToString();
                this.tbZoom_Scroll(this, EventArgs.Empty);
            }
        }

        public Stream GetImageData()
        {
            var data = new MemoryStream();

            // Create a new bitmap.
            Bitmap bmp = this.Image;

            // Lock the bitmap's bits.  
            Rectangle rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
            System.Drawing.Imaging.BitmapData bmpData =
                bmp.LockBits(rect, System.Drawing.Imaging.ImageLockMode.ReadWrite,
                bmp.PixelFormat);

            // Get the address of the first line.
            IntPtr ptr = bmpData.Scan0;

            // Declare an array to hold the bytes of the bitmap.
            int bytes = bmpData.Stride * bmp.Height;
            byte[] rgbValues = new byte[bytes];

            // Copy the RGB values into the array.
            System.Runtime.InteropServices.Marshal.Copy(ptr, rgbValues, 0, bytes);

            for (int i = 0; i < rgbValues.Length; i++)
                data.WriteByte(rgbValues[i]);

            // Unlock the bits.
            bmp.UnlockBits(bmpData);

            return data;
        }

        public ImageVisualizerForm()
        {
            InitializeComponent();
        }

        private void tbZoom_Scroll(object sender, EventArgs e)
        {
            this.lblZoom.Text = "Zoom: " + this.tbZoom.Value.ToString() + "%";
            float percent = this.tbZoom.Value / 100f;
            if (this.pbPreview.Image == null) return;
            this.pbPreview.Size = new Size((int)(this.pbPreview.Image.Width * percent), (int)(this.pbPreview.Image.Height * percent));
        }
    }
}
