﻿/*
Created by: X
http://www.createdbyx.com/

This work is licensed under a Creative Commons Attribution-Noncommercial-Share Alike 2.5 Canada License 
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/. 
Permissions beyond the scope of this license may be available at http://contact.createdbyx.com/.
 * 
 * Very special thanks to Derek Smyth =>http://dotnetslackers.com/Community/blogs/dsmyth/default.aspx
 * His blog post titled "Type Visualizers : Visualizing non-serializable types." helped me in figuring 
 * out how to create a debug visualizer for types that are not marked serializable.
 * http://dotnetslackers.com/Community/blogs/dsmyth/archive/2007/05/29/Type-Visualizers-_3A00_-Visualizing-non_2D00_serializable-types_2E00_.aspx
 * 
*/

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.VisualStudio.DebuggerVisualizers;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using XNAVisies;
using Color = Microsoft.Xna.Framework.Graphics.Color;
using Rectangle = System.Drawing.Rectangle;

[assembly: DebuggerVisualizer(typeof(XNAVisies.Texture2DVisualizer), typeof(Texture2DDebugTypeVisualizer),
                              Target = typeof(Texture2D), Description = "XNAVisies: Texture2D Visualizer")]

namespace XNAVisies
{
    public class Texture2DDebugTypeVisualizer : Microsoft.VisualStudio.DebuggerVisualizers.VisualizerObjectSource
    {
        public Texture2DDebugTypeVisualizer()
            : base()
        {

        }

        public override void GetData(object target, System.IO.Stream outgoingData)
        {
            try
            {
                BinaryWriter writer = new BinaryWriter(outgoingData);

                if (!(target is Texture2D))
                {
                    writer.Write(false);
                    writer.Write("Does not appear to be a valid Texture2D object.");
                    writer = null;
                    return;
                }
                Texture2D tex = (Texture2D)target;

                SurfaceFormat format = tex.Format;
                bool validFormat = this.ValidTextureFormat(format);

                writer.Write(true);
                writer.Write(tex.Width);
                writer.Write(tex.Height);
                writer.Write((int)format);
                writer.Write(validFormat);

                if (!validFormat)
                {
                    writer.Write("The texture does not have a valid texture format.");
                    return;
                }

                // only support valid 360 texture format types
                Color[] colorData;
                byte[] byteData;
                switch (format)
                {
                    case SurfaceFormat.Color:
                    case SurfaceFormat.Bgra1010102:
                    case SurfaceFormat.Rgba32:
                    case SurfaceFormat.Rgba1010102:
                        colorData = new Color[tex.Width * tex.Height];
                        tex.GetData<Color>(colorData);
                        for (int i = 0; i < colorData.Length; i++)
                        {
                            outgoingData.WriteByte(colorData[i].R);
                            outgoingData.WriteByte(colorData[i].G);
                            outgoingData.WriteByte(colorData[i].B);
                            outgoingData.WriteByte(colorData[i].A);
                        }
                        break;
                    case SurfaceFormat.Bgr32:
                    case SurfaceFormat.Rgb32:
                        colorData = new Color[tex.Width * tex.Height];
                        tex.GetData<Color>(colorData);
                        for (int i = 0; i < colorData.Length; i++)
                        {
                            outgoingData.WriteByte(colorData[i].R);
                            outgoingData.WriteByte(colorData[i].G);
                            outgoingData.WriteByte(colorData[i].B);
                            outgoingData.WriteByte(255);
                        }
                        break;
                    case SurfaceFormat.Alpha8:
                        byteData = new byte[tex.Width * tex.Height];
                        tex.GetData<byte>(byteData);
                        for (int i = 0; i < byteData.Length; i++)
                        {
                            outgoingData.WriteByte(0);
                            outgoingData.WriteByte(255);
                            outgoingData.WriteByte(0);
                            outgoingData.WriteByte(byteData[i]);
                        }
                        break;
                    case SurfaceFormat.Single:
                        float[] floatData = new float[tex.Width * tex.Height];
                        tex.GetData<float>(floatData);
                        for (int i = 0; i < floatData.Length; i++)
                        {
                            Color c = new Color(0f, floatData[i], 0f, 255);

                            outgoingData.WriteByte(c.R);
                            outgoingData.WriteByte(c.G);
                            outgoingData.WriteByte(c.B);
                            outgoingData.WriteByte(c.A);
                        }
                        break;
                    case SurfaceFormat.Vector2:
                        Vector2[] v2Data = new Vector2[tex.Width * tex.Height];
                        tex.GetData<Vector2>(v2Data);
                        for (int i = 0; i < v2Data.Length; i++)
                        {
                            Color c = new Color(v2Data[i].X, v2Data[i].Y, 0f, 255);

                            outgoingData.WriteByte(c.R);
                            outgoingData.WriteByte(c.G);
                            outgoingData.WriteByte(c.B);
                            outgoingData.WriteByte(c.A);
                        }
                        break;
                    case SurfaceFormat.Vector4:
                        Vector4[] v4Data = new Vector4[tex.Width * tex.Height];
                        tex.GetData<Vector4>(v4Data);
                        for (int i = 0; i < v4Data.Length; i++)
                        {
                            Color c = new Color(v4Data[i].X, v4Data[i].Y, v4Data[i].Z, v4Data[i].W);

                            outgoingData.WriteByte(c.R);
                            outgoingData.WriteByte(c.G);
                            outgoingData.WriteByte(c.B);
                            outgoingData.WriteByte(c.A);
                        }
                        break;
                    case SurfaceFormat.HalfVector2:
                        break;
                    case SurfaceFormat.HalfVector4:
                        break;
                    case SurfaceFormat.NormalizedAlpha1010102:
                        break;
                }

            }
            catch (Exception ex)
            {
                // MessageBox.Show("An unexpected error occoured!\r\n\r\n" + ex.Message + "\r\n\r\n" + ex.StackTrace, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private bool ValidTextureFormat(SurfaceFormat format)
        {
            // only support valid 360 texture format types
            switch (format)
            {
                case SurfaceFormat.Color:
                case SurfaceFormat.Bgr32:
                case SurfaceFormat.Bgra1010102:
                case SurfaceFormat.Rgba32:
                case SurfaceFormat.Rgb32:
                case SurfaceFormat.Rgba1010102:
                case SurfaceFormat.Alpha8:
                case SurfaceFormat.Single:
                case SurfaceFormat.Vector2:
                case SurfaceFormat.Vector4:
                // case SurfaceFormat.HalfVector2:               // these three formats are not implemented because of my lack
                // case SurfaceFormat.HalfVector4:               // experience working with bit-shifting operators. :(
                //  case SurfaceFormat.NormalizedAlpha1010102:
                    return true;
            }

            return false;
        }

        public override void TransferData(object target, System.IO.Stream incomingData, System.IO.Stream outgoingData)
        {
            base.TransferData(target, incomingData, outgoingData);
        }

        public override object CreateReplacementObject(object target, System.IO.Stream incomingData)
        {
            return base.CreateReplacementObject(target, incomingData);
        }
    }

    // TODO: Add the following to SomeType's definition to see this visualizer when debugging instances of SomeType:
    //
    // [DebuggerVisualizer(typeof(MatrixVisualizer))]
    // [Serializable]
    // public class SomeType
    // {
    // ...
    // }
    //
    /// <summary>
    /// A Visualizer for SomeType.
    /// </summary>
    public class Texture2DVisualizer : DialogDebuggerVisualizer
    {

        protected override void Show(IDialogVisualizerService windowService, IVisualizerObjectProvider objectProvider)
        {
            try
            {
                System.Diagnostics.Debugger.Launch();
                var stream = objectProvider.GetData();


                using (ImageVisualizerForm displayForm = new ImageVisualizerForm())
                {

                    System.IO.BinaryReader reader = new BinaryReader(stream);

                    bool status = reader.ReadBoolean();
                    if (!status)
                    {
                        string err = reader.ReadString();
                        MessageBox.Show(err, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }

                    int width = reader.ReadInt32();
                    int height = reader.ReadInt32();
                    SurfaceFormat format = (SurfaceFormat)reader.ReadInt32();
                    displayForm.Format = format;
                    bool valid = reader.ReadBoolean();

                    if (!valid)
                    {
                        string tmp = reader.ReadString();
                        MessageBox.Show(tmp, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    Bitmap bmp = new Bitmap(width, height, PixelFormat.Format32bppArgb);


                    // Lock the bitmap's bits.  
                    Rectangle rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
                    System.Drawing.Imaging.BitmapData bmpData =
                        bmp.LockBits(rect, System.Drawing.Imaging.ImageLockMode.ReadWrite,
                                     bmp.PixelFormat);

                    // Get the address of the first line.
                    IntPtr ptr = bmpData.Scan0;

                    // Declare an array to hold the bytes of the bitmap.
                    int bytes = bmpData.Stride * bmp.Height;
                    byte[] rgbValues = new byte[bytes];

                    // Copy the RGB values into the array.
                    System.Runtime.InteropServices.Marshal.Copy(ptr, rgbValues, 0, bytes);


                    // copy pixel data into bitmat
                    for (int i = 0; i < rgbValues.Length; i += 4)
                    {
                        rgbValues[i + 2] = reader.ReadByte();
                        rgbValues[i + 1] = reader.ReadByte();
                        rgbValues[i] = reader.ReadByte();
                        rgbValues[i + 3] = reader.ReadByte();
                    }

                    // Copy the RGB values back to the bitmap
                    System.Runtime.InteropServices.Marshal.Copy(rgbValues, 0, ptr, bytes);

                    // Unlock the bits.
                    bmp.UnlockBits(bmpData);


                    // assign the matrix
                    displayForm.Image = bmp;

                    if (windowService.ShowDialog(displayForm) == DialogResult.OK)
                    {
                        try
                        {
                            // update object values
                            //objectProvider.ReplaceObject(displayForm.Color);
                        }
                        catch (Exception)
                        {
                            MessageBox.Show("Could not change the texture data.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("An unexpected error occoured!\r\n\r\n" + ex.Message + "\r\n\r\n" + ex.StackTrace, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // TODO: Add the following to your testing code to test the visualizer:
        //
        // MatrixVisualizer.TestShowVisualizer(new SomeType());
        //
        /// <summary>
        /// Tests the visualizer by hosting it outside of the debugger.
        /// </summary>
        /// <param name="objectToVisualize">The object to display in the visualizer.</param>
        public static void TestShowVisualizer(object objectToVisualize)
        {
            VisualizerDevelopmentHost visualizerHost = new VisualizerDevelopmentHost(objectToVisualize, typeof(Texture2DVisualizer));
            visualizerHost.ShowVisualizer();
        }
    }
}
